from flask import Flask, render_template, jsonify, request
from copy import copy
import uuid
app = Flask(__name__)

# Vitals
vitalsHeartrate = None
vitalsOxygen = None
vitalsBodytemp = None

@app.route('/api/vitals/heartrate')
def getVitalsHeartrate():
	returnArr = [ { 'heartrate': vitalsHeartrate } ]
	return jsonify(returnArr)

@app.route('/api/vitals/heartrate', methods=['POST'])
def setVitalsHeartrate():
	global vitalsHeartrate
	json = request.get_json()
	try:
		if not authenticate(json['uuid'], '/api/vitals/heartrate'):
			return 'Forbidden.', 403

		vitalsHeartrate = json['heartrate']
	except:
		return 'Incorrect usage.\nUsage: { heartrate: INT, uuid: STRING }\n', 400
	return '', 204

@app.route('/api/vitals/oxygen')
def getVitalsOxygen():
	returnArr = [ { 'oxygen': vitalsOxygen } ]
	return jsonify(returnArr)

@app.route('/api/vitals/oxygen', methods=['POST'])
def setVitalsOxygen():
	global vitalsOxygen
	json = request.get_json()
	try:
		if not authenticate(json['uuid'], '/api/vitals/oxygen'):
			return 'Forbidden.', 403

		vitalsOxygen = json['oxygen']
	except:
		return 'Incorrect usage.\nUsage: { oxygen: INT }\n', 400
	return '', 204

@app.route('/api/vitals/bodytemp')
def getVitalsBodytemp():
	returnArr = [ { 'bodytemp': vitalsBodytemp } ]
	return jsonify(returnArr)

@app.route('/api/vitals/bodytemp', methods=['POST'])
def setVitalsBodytemp():
	global vitalsBodytemp
	json = request.get_json()
	try:
		if not authenticate(json['uuid'], '/api/vitals/bodytemp'):
			return 'Forbidden.', 403

		vitalsBodytemp = json['bodytemp']
	except:
		return 'Incorrect usage.\nUsage: { bodytemp: FLOAT }\n', 400
	return '', 204

@app.route('/api/vitals')
def getVitals():
	returnArr = [ { 'heartrate': vitalsHeartrate, 'oxygen': vitalsOxygen, 'bodytemp': vitalsBodytemp } ]
	return jsonify(returnArr)

@app.route('/api/vitals', methods=['POST'])
def setVitals():
	global vitalsHeartrate
	global vitalsOxygen
	global vitalsBodytemp

	json = request.get_json()
	try:
		if not authenticate(json['uuid'], '/api/vitals'):
			return 'Forbidden.', 403
		# This is a bit ugly but its just how I'm checking that everything is there without setting variables if the json is incorrect
		tempH = json['heartrate']
		tempO = json['oxygen']
		tempB = json['bodytemp']
		vitalsHeartrate = tempH
		vitalsOxygen = tempO
		vitalsBodytemp = tempB
	except:
		return 'Incorrect usage.\nUsage: { heartrate: INT, oxygen: INT, bodytemp: FLOAT }\n', 400
	return '', 204

# Fitness
fitnessSteps = None

@app.route('/api/fitness/steps')
def getSteps():
	return getFitness() # This is the same for now.

@app.route('/api/fitness/steps', methods=['POST'])
def setFitnessSteps():
	global fitnessSteps
	json = request.get_json()
	try:
		if not authenticate(json['uuid'], '/api/fitness/steps'):
			return 'Forbidden.', 403

		vitalsBodytemp = json['steps']
	except:
		return 'Incorrect usage.\nUsage: { steps: INT }\n', 400
	return '', 204


@app.route('/api/fitness')
def getFitness():
	returnArr = [ { 'steps': fitnessSteps } ]
	return jsonify(returnArr)

@app.route('/api/fitness', methods=['POST'])
def setFitness():
	global fitnessSteps
	json = request.get_json()
	try:
		if not authenticate(json['uuid'], '/api/fitness'):
			return 'Forbidden.', 403

		fitnessSteps = json['steps']
	except:
		return 'Incorrect usage.\nUsage: { steps: INT }\n', 400
	return '', 204


# Cyberware management
cyberware = []
newCyberwareTemplate = { "uuid": None, "name": None, "hotpluggable": False, "lastMalfunction": None, "frontendAwknowledge": False, "canSet": [], "battery": None }


# This makes the system aware of a new piece of hardware.
# While, for the most part, not required due to the design of this project,
#  it's handy for error reporting and communication between hardware and
#  the end user.
#
# Arguments: { name: STRING, hotpluggable: BOOL, canSet: ARRAY }
# Returns:   { uuid: INT }
# name: A human-readable name
# hotpluggable argument: Used for removable modules known as shards.
#
# uuid: Returns a uuid for the hardware.
@app.route('/api/cyberware/add', methods=['POST'])
def addCyberware():
	global cyberware
	json = request.get_json()

	try:
		tempName = json['name']
		tempHotpluggable = json['hotpluggable']
		tempNewCyberware = copy(newCyberwareTemplate)

		tempNewCyberware['uuid'] = str(uuid.uuid4())
		tempNewCyberware['name'] = tempName
		tempNewCyberware['hotpluggable'] = tempHotpluggable

		cyberware.append(tempNewCyberware)
	except:
		return 'Incorrect usage.\nUsage: { name: STRING, hotpluggable: BOOL, canSet: ARRAY }\n', 400
	return jsonify([ { "uuid": tempNewCyberware['uuid'] } ]), 200

# Arguments: { uuid: INT }
@app.route('/api/cyberware/remove', methods=['POST'])
def removeCyberware():
	global cyberware
	json = request.get_json()

	try:
		desiredId = json['uuid']
		i = 0
		for c in cyberware:
			if c['uuid'] == desiredId:
				del cyberware[i]
				return '', 204
			i = i + 1
	except:
		return 'Incorrect usage.\nUsage: { uuid: STRING }\n', 400
	return 'UUID Invalid\n', 400

# Returns: { uuid: STRING, name: STRING, hotpluggable: BOOL, lastMalfunction: STRING, frontendAwknowledge: BOOL }
# uuid:               Unique identifier of the hardware
# name:               Human-readable name
# hotpluggable:       Hardware can be removed during runtime.
# lastMalfunction:    A string with information on the last malfunction.
# frontendAwknowlege: A tool for the frontend to keep track of whether it's aware of the hardware.
# TODO: If this method is to exist, it needs to return everything WITHOUT UUID's.
@app.route('/api/cyberware')
def getCyberware():
	return 'Not implemented', 501

	#if not authenticate(json['uuid'], '/api/cyberware'):
	#		return 'Forbidden.', 403
	#return jsonify(cyberware)

@app.route('/api/cyberware/get', methods=['POST'])
def getCyberwareSpecific():
	json = request.get_json()

	try:
		desiredId = json['uuid']

		requestedCyberware = getCyberwareHelper(desiredId)

		if (requestedCyberware != None):
			return jsonify(requestedCyberware), 200
	except:
		return 'Incorrect usage.\nUsage: { uuid: STRING }\n', 400
	return 'UUID Invalid\n', 400

# Arguments { uuid: INT }
@app.route('/api/cyberware/battery')
def getCyberwareBattery():
	json = request.get_json()

	try:
		desiredId = json['uuid']

		requestedCyberware = getCyberwareHelper(desiredId)

		if (requestedCyberware != None):
			return jsonify([ { "battery": requestedCyberware['battery'] } ]), 200
	except:
		return 'Incorrect usage.\nUsage: { uuid: STRING }\n', 400
	return 'UUID Invalid\n', 400

# Arguments { uuid: STRING, battery: INT }
@app.route('/api/cyberware/battery', methods=['POST'])
def setCyberwareBattery():
	json = request.get_json()

	try:
		desiredId = json['uuid']
		battery = json['battery']

		requestedCyberware = getCyberwareHelper(desiredId)

		if (requestedCyberware != None):
			requestedCyberware['battery'] = battery
			return '', 204
	except:
		return 'Incorrect usage.\nUsage: { battery: INT, uuid: STRING }\n', 400
	return 'UUID Invalid\n', 400

# Arguments { uuid: STRING }
@app.route('/api/cyberware/reset_malfunction', methods=['POST'])
def resetCyberwareMalfunction():
	json = request.get_json()

	try:
		desiredId = json['uuid']

		requestedCyberware = getCyberwareHelper(desiredId)

		if (requestedCyberware != None):
			requestedCyberware['lastMalfunction'] = None
			return '', 204
	except:
		return 'Incorrect usage.\nUsage: { uuid: STRING }\n', 400
	return 'UUID Invalid\n', 400

def getCyberwareHelper(uuid):
	i = 0
	for c in cyberware:
		if c['uuid'] == desiredId:
			return c
			break
	return None

# Environment data
environmentTemperature = None
environmentHumidity = None

@app.route('/api/environment')
def getEnvironment():
	returnArr = [ { 'temperature': environmentTemperature, 'humidity': environmentHumidity } ]
	return jsonify(returnArr), 200

# Environment//Temperature
@app.route('/api/environment/temperature')
def getEnvironmentTemperature():
	returnArr = [ { 'temperature': environmentTemperature } ]
	return jsonify(returnArr), 200

@app.route('/api/environment/temperature', methods=['POST'])
def setEnvironmentTemperature():
	global environmentTemperature
	json = request.get_json()

	try:
		tempTemperature = json['temperature']

		environmentTemperature = tempTemperature
	except:
		 return 'Incorrect usage.\nUsage: { temperature: INT, uuid: STRING }\n', 400
	return '', 204

# Environment//Humidity
@app.route('/api/environment/humidity')
def getEnvironmentHumidity():
	returnArr = [ { 'humidity': environmentHumidity } ]
	return jsonify(returnArr), 200

@app.route('/api/environment/humidity', methods=['POST'])
def setEnvironmentHumidity():
	global environmentHumidity
	json = request.get_json()

	try:
		tempHumidity = json['humidity']

		environmentHumidity = tempHumidity
	except:
		return 'Incorrect usage.\nUsage: { humidity: INT, uuid: STRING }\n', 400
	return '', 204

# Authentication method
# This authorizes the given UUID to determine whether the request is
#  allowed to set the requested endpoint.
# This is ONLY used for PUSH requests currently.
def authenticate(uuid, endpoint):
	for c in cyberware: # UUID Match
		if c['uuid'] == uuid:
			requestedHardware = c
			break

	for e in requestedHardware['canSet']: # Endpoint match
		if e == endpoint:
			return True
	return False


@app.route('/')
def uiindex():
	return render_template('index.html')
